[cylc]
   UTC mode = True
   [[reference test]]
       required run mode = live
       live mode suite timeout=PT1M
[scheduling]
    initial cycle point = 20140107
    final cycle point = 20140118
    runahead limit = P1D # enforce sequential running
    [[dependencies]]
        [[[20140110T06/P3D]]]
            # daily cycling *with a non-zero hour* is a more stringent test
            graph = "a"
[runtime]
    [[a]]
        script = "sleep 1"

[visualization]
    initial cycle point = 20140107
    final cycle point = 20140118
