#!jinja2

[scheduling]
    [[dependencies]]
        graph = hello_world
[runtime]
    [[hello_world]]
    {% if cylc_compat_mode is defined and cylc_compat_mode == '6' %}
        command scripting = echo "Hello World!"
    {% else %}
        script = echo "Hello World!"
    {% endif %}
