[cylc]
    cycle point format = %Y
    [[events]]
        inactivity = PT1M
        abort on inactivity = True
        startup handler = cylc release '%(suite)s'
[scheduling]
    initial cycle point = 2016
    final cycle point = 2020
    [[dependencies]]
        [[[P1Y]]]
            graph = t1 & t2 &t3
[runtime]
    [[t1, t2, t3]]
        script = true
