[cylc]
    [[reference test]]
        required run mode = live
        live mode suite timeout = PT1M
[scheduling]
    initial cycle point = 20100101T00
    final cycle point = 20100102T00
    [[dependencies]]
        [[[P1D]]]
            graph = "daily"
        [[[P1M]]]
            graph = "monthly"
        [[[P1Y]]]
            graph = "yearly"
[runtime]
    [[root]]
        script = "true"
