[scheduling]
    initial cycle point = 1
    cycling mode = integer
    [[dependencies]]
        [[[P1!3]]]
            graph = foo[-P1] => foo
[runtime]
    [[foo]]
[visualization]
    number of cycle points = 4
