[cylc]
   UTC mode = True
   [[events]]
       abort on stalled = True
   [[reference test]]
       live mode suite timeout = PT1M
       required run mode = live
[scheduling]
    [[dependencies]]
        graph = t1 => t2
[runtime]
    [[t1]]
        script=true
        [[[outputs]]]
            hello = Hello World
            greet = Greet World
    [[t2]]
        script = """
LOG="${CYLC_SUITE_LOG_DIR}/log"
cylc reset --output='!hello' --output='!Greet World' "${CYLC_SUITE_NAME}" 't1.1'
while ! grep -qF -e '[t1.1] -reset output to incomplete: hello' "${LOG}"; do
    sleep 1  # make sure reset completes
done
while ! grep -qF '[t1.1] -reset output to incomplete: Greet World' "${LOG}"; do
    sleep 1  # make sure reset completes
done
cylc show "${CYLC_SUITE_NAME}" 't1.1' >"${CYLC_SUITE_RUN_DIR}/cylc-show.out"
"""
        [[[job]]]
            execution time limit = PT30S
