#!jinja2

# A suite to test the validation performance of cylc when dealing with family
# triggers.

# Implemented using jinja2 rather than parameterized tasks to allow for
# profiling with older cylc versions.

{% if not batch_system is defined %}
    {% set batch_system = 'background' %}
{% endif %}
{% if not no_members is defined %}
    {% set no_members = 50 %}
{% endif %}

[scheduling]
    [[dependencies]]
        graph = FAM1:succeed-all => FAM2
[runtime]
    [[root]]
{% if cylc_compat_mode is defined and cylc_compat_mode == '6' %}
        command scripting = true
        [[[job submission]]]
            method = {{ batch_system }}
{% else %}
        script = true
        [[[job]]]
            batch system = {{ batch_system }}
{% endif %}
    [[FAM1]]
    [[FAM2]]
{% for member in range(1,no_members|int) %}
    [[fam1_member_{{member}}]]
        inherit = FAM1
{% endfor %}
{% for member in range(1,no_members|int) %}
    [[fam2_member_{{member}}]]
        inherit = FAM2
{% endfor %}
