[cylc]
   [[reference test]]
       required run mode = live
       live mode suite timeout=PT1M
[scheduling]
    [[dependencies]]
        graph = "a => b => c"
[runtime]
    [[a,c]]
        script = "true"
    [[b]]
        script = "cylc reload $CYLC_SUITE_NAME -f; sleep 5"
