#!Jinja2

[cylc]
    UTC mode = True

[scheduling]
    initial cycle point = 01231212T1212
    [[dependencies]]
        [[[R1]]]
            graph = strftime => pad => end
[runtime]
    [[strftime]]
        script = test $TEST_1 == '00'; test $TEST_2 == '30'
        [[[environment]]]
            TEST_1 = {{ '2000' | strftime('%H') }}
            TEST_2 = {{ '12-30-2000' | strftime('%d', '%m-%d-%Y') }}
    [[pad]]
        script = test {{ 42 | pad(3, 0) }} == '042'
    [[end]]
        script = true
