[cylc]
    [[events]]
        abort on stalled = True
[scheduling]
    initial cycle point = 1
    cycling mode = integer
    [[dependencies]]
        [[[R1/+P1]]]
            graph = start
        [[[R//P1]]]
            graph = start[2] => foo
[runtime]
    [[root]]
        script = true
