[cylc]
   UTC mode = True
   [[reference test]]
       required run mode = live
       live mode suite timeout = PT1M
[scheduling]
    initial cycle point = 2010-02
    final cycle point = 2010-08
    runahead limit = P1D # Enforce sequential running
    [[dependencies]]
        [[[2010-01/P3M]]]
            graph = "a"
[runtime]
    [[a]]
        script = "sleep 1"
