[meta]
    title = "Hello, Goodbye, Suicide"
[scheduling]
    [[dependencies]]
        graph = """hello => goodbye
           goodbye:fail => really_goodbye
           goodbye => !really_goodbye
           really_goodbye => !goodbye"""
[runtime]
    [[hello]]
        script = "sleep 10; echo Hello World!"
    [[goodbye]]
        script = """
sleep 10
echo Goodbye ... oops!
/bin/false # FAIL"""
