[cylc]
    UTC mode = True
[scheduling]
    initial cycle point = 20140101
    final cycle point = 20140102T12
    [[dependencies]]
        [[[ R1/+PT10M ]]]
            graph = "foo => bar"
[runtime]
    [[root]]
        script = true
[visualization]
    initial cycle point = 20140101
    final cycle point = 20140102T12
