[cylc]
    UTC mode = True
[scheduling]
    initial cycle point = 20131225T1200Z
    final cycle point = 20131225T1200Z
    [[dependencies]]
        [[[ T12 ]]]
            graph = """
                (a[-P1D]:fail | b[-P1D]:fail | c[-P1D]:fail) => d
                a & b & c  # Implied by implicit cycling now...
            """
[runtime]
    [[root]]
        script = sleep 1
