[cylc]
    UTC mode = True
    cycle point format = CCYY-MM-DD
    [[reference test]]
        live mode suite timeout = PT1M

[scheduling]
    initial cycle point = 20100101

[[dependencies]]
    [[[R1]]]
        graph = foo

[runtime]
    [[foo]]
        script = true
