[cylc]
    cycle point format = %Y
    [[events]]
        inactivity = PT1M
        abort on inactivity = True
[scheduling]
    initial cycle point = 2010
    final cycle point = 2012
    [[dependencies]]
        [[[P1Y]]]
            graph = t1[-P1Y] => t1
[runtime]
    [[t1]]
        script = true
