#!Jinja2
[cylc]
    UTC mode = True
    [[events]]
        abort on stalled = True
        abort on inactivity = True
        inactivity = PT1M
    [[reference test]]
        required run mode = live
        live mode suite timeout = PT1M
[scheduling]
    [[dependencies]]
        graph = t1
[runtime]
    [[t1]]
        script = """
grep -F -q "CYLC_SUITE_HOST={{MY_HOST_IP}}" "${CYLC_SUITE_RUN_DIR}/.service/contact"
"""
        [[[job]]]
            execution time limit = PT30S
