[cylc]
   UTC mode = True
   [[reference test]]
       required run mode = live
       live mode suite timeout=PT1M
[scheduling]
    initial cycle point = 20100101T00
    final cycle point = 20100102T00
    [[dependencies]]
        [[[T00]]]
            graph = "a[-P1D] & b => a"
[runtime]
    [[a, b]]
        script = "sleep 1"
