#!/bin/bash
# THIS FILE IS PART OF THE CYLC SUITE ENGINE.
# Copyright (C) 2008-2017 NIWA
# 
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#-------------------------------------------------------------------------------
# jinja2 test cylc-provided functions.
. $(dirname $0)/test_header
#-------------------------------------------------------------------------------
set_test_number 4
#-------------------------------------------------------------------------------
install_suite "${TEST_NAME_BASE}" "${TEST_NAME_BASE}"
#-------------------------------------------------------------------------------
# raise(message, error_type='Error')
TEST_NAME="${TEST_NAME_BASE}-raise-validate"
run_fail "${TEST_NAME}" cylc validate "${SUITE_NAME}"
cmp_ok "${TEST_NAME}.stderr" << '__ERR__'
Jinja2 Error: Some Exception
__ERR__
#-------------------------------------------------------------------------------
# assert(logical, message)
TEST_NAME="${TEST_NAME_BASE}-assert-validate"
run_fail "${TEST_NAME}" cylc validate "${SUITE_NAME}" -s 'ANSWER=43'
cmp_ok "${TEST_NAME}.stderr" << '__ERR__'
Jinja2 Assertation Error: Reality check needed
__ERR__
#-------------------------------------------------------------------------------
purge_suite "${SUITE_NAME}"
