[meta]
    title = "Job script: directives test for SGE"
[scheduling]
    [[dependencies]]
        graph = foo
[runtime]
    [[foo]]
        [[[job]]]
            batch system = sge
            execution time limit = PT10M
        [[[directives]]]
            -V =
            -q = queuename
            -l = s_vmem=1G,s_cpu=60
