#!jinja2
[cylc]
   abort if any task fails = True
   [[events]]
       abort on inactivity = True
       inactivity = PT2M
   [[reference test]]
       required run mode = live
       live mode suite timeout = PT30S

[scheduling]
    [[dependencies]]
        graph = foo

[runtime]
    [[foo]]
        script = """
if [[ "${CYLC_TASK_SUBMIT_NUMBER}" == '1' ]]; then
    sleep 10
fi
"""
        [[[job]]]
            batch system = {{CYLC_TEST_BATCH_SYS}}
            execution time limit = PT5S
            execution retry delays = PT0S
