#!Jinja2
[cylc]
    [[reference test]]
        expected task failures = foo.1
    [[events]]
        abort on stalled = True
[scheduling]
    [[dependencies]]
        graph = foo
[runtime]
    [[foo]]
        script = """
test "${CYLC_TASK_SUBMIT_NUMBER}" -ge 2
print 'Kornflakes?'
"""
        [[[job]]]
            shell = {{KSH}}
            execution retry delays = PT0S
