#!jinja2

[meta]
    title = "polls for success and failure tasks in another suite"
[cylc]
    [[reference test]]
        live mode suite timeout = PT1M
[scheduling]
    [[dependencies]]
        graph = "l-good<{{UPSTREAM}}::good-stuff> & lbad<{{UPSTREAM}}::bad:fail>"
[runtime]
    [[l-good,lbad]]
        [[[suite state polling]]]
            interval = PT2S
            max-polls = 20
