[cylc]
   UTC mode = True
   [[reference test]]
       required run mode = live
       live mode suite timeout=PT1M
[scheduling]
    initial cycle point = 2011
    final cycle point = 2018
    runahead limit = P1D
    [[dependencies]]
        [[[2010/P3Y]]]
            graph = "a"
[runtime]
    [[a]]
        script = "sleep 1"
